{% load i18n static %}

<div id="user-tools">
    <a href="{% url 'admin:Add' %}">Add ➕</a> &nbsp; &nbsp;
    <a href="{% url 'Home' %}">Snapshots</a> |
    <a href="/admin/core/tag/">Tags</a> |
    <a href="/admin/core/archiveresult/?o=-1">Log</a> &nbsp; &nbsp;
    <a href="{% url 'Docs' %}">Docs</a> | 
    <a href="{% url 'public-index' %}">Public</a> | 
    <a href="/admin/">Admin</a>
     &nbsp; &nbsp;
    {% if user.is_authenticated %}
        {% block welcome-msg %}
            {% trans 'User' %}
            <strong>{% firstof user.get_short_name user.get_username %}</strong> &nbsp; &nbsp;
        {% endblock %}
        {% block userlinks %}
            {% if user.has_usable_password %}
                <a href="{% url 'admin:password_change' %}">Account</a> /
            {% endif %}
            <a href="{% url 'admin:logout' %}">{% trans 'Log out' %}</a>
        {% endblock %}
    {% else %}
        <a href="{% url 'admin:login' %}">{% trans 'Log in' %}</a>
    {% endif %}
</div>
