header {
    font-family: "Roboto","Lucida Grande","DejaVu Sans","Bitstream Vera Sans",Verdana,Arial,sans-serif;
    font-size: 13px;
    color: white;
    height: 30px;
}
.header-top {
  color: white;
}

.dashboard #content {
  width: 100%;
  margin-right: 0px;
  margin-left: 0px;
}
#submit {
  border: 1px solid rgba(0, 0, 0, 0.2);
  padding: 10px;
  border-radius: 4px;
  background-color: #f5dd5d;
  color: #333;
  font-size: 18px;
  font-weight: 800;
}
#add-form button[role="submit"]:hover {
  background-color: #e5cd4d;
}
#add-form label {
  display: block;
  font-size: 16px;
}
#add-form textarea {
  width: 100%;
  min-height: 300px;
}
#delay-warning div {
  border: 1px solid red;
  border-radius: 4px;
  margin: 10px;
  padding: 10px;
  font-size: 15px;
  background-color: #8bc34a;
}
#stdout {
  background-color: #fbfbfb;
  padding: 10px 10px;
  border-radius: 4px;
  white-space: normal;
}
ul#id_depth {
  list-style-type: none;
  padding: 0;
}

@keyframes spin {
  0% {
    transform: rotate(0deg);
  }
  100% {
    transform: rotate(360deg);
  }
}

.loader {
  border: 16px solid #f3f3f3; /* Light grey */
  border-top: 16px solid #3498db; /* Blue */
  border-radius: 50%;
  width: 30px;
  height: 30px;
  box-sizing: border-box;
  animation: spin 2s linear infinite;
}


textarea, select {
  border-radius: 4px;
  border: 2px solid #004882;
  box-shadow:  4px 4px 4px rgba(0,0,0,0.02);
  width: 100%;
}

select option:not(:checked) {
  border: 1px dashed rgba(10,200,20,0.12);
}
select option:checked {
  border: 1px solid green;
  background-color: green;
  color: green;
}

