# Generated by Django 3.0.7 on 2020-06-25 15:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='snapshot',
            name='timestamp',
            field=models.CharField(default=None, max_length=32, null=True),
        ),
    ]
