# Generated by Django 3.1.3 on 2021-01-05 14:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0007_archiveresult'),
    ]

    operations = [
        migrations.AlterField(
            model_name='archiveresult',
            name='cmd_version',
            field=models.CharField(blank=True, default=None, max_length=32, null=True),
        ),
    ]
